@@ -24,13 +24,25 @@
 #include "pyi_main.h"
 #include "pyi_utils.h"
 
+#ifndef HAVE_WCSDUP
+static wchar_t
+*wcsdup(const wchar_t *s)
+{
+    size_t len = wcslen(s) + 1;
+    wchar_t *new_s = malloc(len * sizeof(wchar_t));
+    if (new_s) {
+        wcscpy(new_s, s);
+    }
+    return new_s;
+}
+#endif
 
 /*
  * Clean up and free the PyiRuntimeOptions structure created by
  * pyi_config_parse_runtime_options(). No-op if passed a NULL pointer.
  */
 void
-pyi_runtime_options_free(PyiRuntimeOptions *options)
+pyi_runtime_options_free(struct PyiRuntimeOptions *options)
 {
     int i;
 
@@ -137,18 +149,18 @@ _pyi_match_and_parse_xflag(const char *flag, const char *name, int *dest_var)
  * Allocate the PyiRuntimeOptions structure and populate it based on
  * options found in the PKG archive.
  */
-PyiRuntimeOptions *
-pyi_runtime_options_read(const PYI_CONTEXT *pyi_ctx)
+struct PyiRuntimeOptions *
+pyi_runtime_options_read(const struct PYI_CONTEXT *pyi_ctx)
 {
-    PyiRuntimeOptions *options;
-    const ARCHIVE *archive = pyi_ctx->archive;
-    const TOC_ENTRY *toc_entry;
+    struct PyiRuntimeOptions *options;
+    const struct ARCHIVE *archive = pyi_ctx->archive;
+    const struct TOC_ENTRY *toc_entry;
     int num_wflags = 0;
     int num_xflags = 0;
     int failed = 0;
 
     /* Allocate the structure */
-    options = calloc(1, sizeof(PyiRuntimeOptions));
+    options = calloc(1, sizeof(struct PyiRuntimeOptions));
     if (options == NULL) {
         return options;
     }
@@ -209,11 +221,14 @@ pyi_runtime_options_read(const PYI_CONTEXT *pyi_ctx)
 
     /* Collect Wflags and Xflags for pass-through */
 
-    /* Allocate - calloc should be safe to call with num = 0 (and returns
-     * a non-NULL address that should be safe to free) */
+    /* Allocate - calloc should be safe to call with num = 0.
+     * On most platforms, when called with num = 0, calloc returns a
+     * non-NULL address that should be safe to free. On AIX, though,
+     * it returns NULL (unless _LINUX_SOURCE_COMPAT is defined, but we
+     * cannot have that defined together with _ALL_SOURCE). */
     options->wflags = calloc(num_wflags, sizeof(wchar_t *));
     options->xflags = calloc(num_xflags, sizeof(wchar_t *));
-    if (options->wflags == NULL || options->xflags == NULL) {
+    if ((num_wflags && options->wflags == NULL) || (num_xflags && options->xflags == NULL)) {
         failed = 1;
         goto end;
     }
@@ -266,7 +281,7 @@ end:
  * is used, which internally calls Py_DecodeLocale.
  */
 static int
-_pyi_pyconfig_set_string(PyConfig *config, wchar_t **dest_field, const char *str)
+_pyi_pyconfig_set_string(PyConfig *config, wchar_t **dest_field, const char *str, const struct DYLIB_PYTHON *dylib_python)
 {
     PyStatus status;
 
@@ -276,35 +291,44 @@ _pyi_pyconfig_set_string(PyConfig *config, wchar_t **dest_field, const char *str
     if (!str_w) {
         return -1;
     }
-    status = PI_PyConfig_SetString(config, dest_field, str_w);
+    status = dylib_python->PyConfig_SetString(config, dest_field, str_w);
     free(str_w);
 #else
-    status = PI_PyConfig_SetBytesString(config, dest_field, str);
+    status = dylib_python->PyConfig_SetBytesString(config, dest_field, str);
 #endif
 
-    return PI_PyStatus_Exception(status) ? -1 : 0;
+    return dylib_python->PyStatus_Exception(status) ? -1 : 0;
 }
 
 
+/* Helper for creating ID from python version and flags value */
+#define _MAKE_VERSION_ID(version, flags) (version << 1 | flags)
+
 /*
- * Allocate the PyConfig structure, based on the python version.
+ * Allocate the PyConfig structure, based on the python version and
+ * build flags.
  */
 PyConfig *
-pyi_pyconfig_create(int python_version)
+pyi_pyconfig_create(const struct PYI_CONTEXT *pyi_ctx)
 {
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
+    const int version_id = _MAKE_VERSION_ID(dylib_python->version, pyi_ctx->nogil_enabled);
+
     /* Macro to avoid manual code repetition. */
-    #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
-    case PY_VERSION: { \
+    #define _IMPL_CASE(PY_VERSION, PY_FLAGS, PYCONFIG_IMPL) \
+    case _MAKE_VERSION_ID(PY_VERSION, PY_FLAGS): { \
         return (PyConfig *)calloc(1, sizeof(PYCONFIG_IMPL)); \
     }
     /* Macro end */
 
-    switch (python_version) {
-        _IMPL_CASE(308, PyConfig_v38)
-        _IMPL_CASE(309, PyConfig_v39)
-        _IMPL_CASE(310, PyConfig_v310)
-        _IMPL_CASE(311, PyConfig_v311)
-        _IMPL_CASE(312, PyConfig_v312)
+    switch (version_id) {
+        _IMPL_CASE(308, 0, PyConfig_v38)
+        _IMPL_CASE(309, 0, PyConfig_v39)
+        _IMPL_CASE(310, 0, PyConfig_v310)
+        _IMPL_CASE(311, 0, PyConfig_v311)
+        _IMPL_CASE(312, 0, PyConfig_v312)
+        _IMPL_CASE(313, 0, PyConfig_v313)
+        _IMPL_CASE(313, 1, PyConfig_v313_GIL_DISABLED)
         default: {
             break;
         }
@@ -316,13 +340,19 @@ pyi_pyconfig_create(int python_version)
 }
 
 /*
- * Clean up and free the PyConfig structure.
+ * Clean up and free the PyConfig structure. No-op if passed a NULL pointer.
  */
 void
-pyi_pyconfig_free(PyConfig *config)
+pyi_pyconfig_free(PyConfig *config, const struct PYI_CONTEXT *pyi_ctx)
 {
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
+
+    if (config == NULL) {
+        return;
+    }
+
     /* Clear the fields that PyConfig API allocated */
-    PI_PyConfig_Clear(config);
+    dylib_python->PyConfig_Clear(config);
 
     /* Free the allocated structure itself; was allocated using calloc
      * in pyi_pyconfig_create(). */
@@ -333,25 +363,30 @@ pyi_pyconfig_free(PyConfig *config)
  * Set program name. Used to set sys.executable, and in early error messages.
  */
 int
-pyi_pyconfig_set_program_name(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
+pyi_pyconfig_set_program_name(PyConfig *config, const struct PYI_CONTEXT *pyi_ctx)
 {
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
+    const int version_id = _MAKE_VERSION_ID(dylib_python->version, pyi_ctx->nogil_enabled);
+
     /* Macro to avoid manual code repetition. */
-    #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
-    case PY_VERSION: { \
+    #define _IMPL_CASE(PY_VERSION, PY_FLAGS, PYCONFIG_IMPL) \
+    case _MAKE_VERSION_ID(PY_VERSION, PY_FLAGS): { \
         PYCONFIG_IMPL *config_impl = (PYCONFIG_IMPL *)config; \
-        if (_pyi_pyconfig_set_string(config, &config_impl->program_name, pyi_ctx->executable_filename) < 0) { \
+        if (_pyi_pyconfig_set_string(config, &config_impl->program_name, pyi_ctx->executable_filename, dylib_python) < 0) { \
             return -1; \
         } \
         return 0; \
     }
     /* Macro end */
 
-    switch (pyi_ctx->archive->python_version) {
-        _IMPL_CASE(308, PyConfig_v38)
-        _IMPL_CASE(309, PyConfig_v39)
-        _IMPL_CASE(310, PyConfig_v310)
-        _IMPL_CASE(311, PyConfig_v311)
-        _IMPL_CASE(312, PyConfig_v312)
+    switch (version_id) {
+        _IMPL_CASE(308, 0, PyConfig_v38)
+        _IMPL_CASE(309, 0, PyConfig_v39)
+        _IMPL_CASE(310, 0, PyConfig_v310)
+        _IMPL_CASE(311, 0, PyConfig_v311)
+        _IMPL_CASE(312, 0, PyConfig_v312)
+        _IMPL_CASE(313, 0, PyConfig_v313)
+        _IMPL_CASE(313, 1, PyConfig_v313_GIL_DISABLED)
         default: {
             break;
         }
@@ -366,22 +401,27 @@ pyi_pyconfig_set_program_name(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
  * Set python home directory. Used to set sys.prefix.
  */
 int
-pyi_pyconfig_set_python_home(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
+pyi_pyconfig_set_python_home(PyConfig *config, const struct PYI_CONTEXT *pyi_ctx)
 {
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
+    const int version_id = _MAKE_VERSION_ID(dylib_python->version, pyi_ctx->nogil_enabled);
+
     /* Macro to avoid manual code repetition. */
-    #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
-    case PY_VERSION: { \
+    #define _IMPL_CASE(PY_VERSION, PY_FLAGS, PYCONFIG_IMPL) \
+    case _MAKE_VERSION_ID(PY_VERSION, PY_FLAGS): { \
         PYCONFIG_IMPL *config_impl = (PYCONFIG_IMPL *)config; \
-        return _pyi_pyconfig_set_string(config, &config_impl->home, pyi_ctx->application_home_dir); \
+        return _pyi_pyconfig_set_string(config, &config_impl->home, pyi_ctx->application_home_dir, dylib_python); \
     }
     /* Macro end */
 
-    switch (pyi_ctx->archive->python_version) {
-        _IMPL_CASE(308, PyConfig_v38)
-        _IMPL_CASE(309, PyConfig_v39)
-        _IMPL_CASE(310, PyConfig_v310)
-        _IMPL_CASE(311, PyConfig_v311)
-        _IMPL_CASE(312, PyConfig_v312)
+    switch (version_id) {
+        _IMPL_CASE(308, 0, PyConfig_v38)
+        _IMPL_CASE(309, 0, PyConfig_v39)
+        _IMPL_CASE(310, 0, PyConfig_v310)
+        _IMPL_CASE(311, 0, PyConfig_v311)
+        _IMPL_CASE(312, 0, PyConfig_v312)
+        _IMPL_CASE(313, 0, PyConfig_v313)
+        _IMPL_CASE(313, 1, PyConfig_v313_GIL_DISABLED)
         default: {
             break;
         }
@@ -402,25 +442,30 @@ pyi_pyconfig_set_python_home(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
  * sys.path.
  */
 static int
-_pyi_pyconfig_set_module_search_paths(PyConfig *config, int python_version, int num_paths, wchar_t **paths)
+_pyi_pyconfig_set_module_search_paths(PyConfig *config, const struct PYI_CONTEXT *pyi_ctx, int num_paths, wchar_t **paths)
 {
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
+    const int version_id = _MAKE_VERSION_ID(dylib_python->version, pyi_ctx->nogil_enabled);
+
     /* Macro to avoid manual code repetition. */
-    #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
-    case PY_VERSION: { \
+    #define _IMPL_CASE(PY_VERSION, PY_FLAGS, PYCONFIG_IMPL) \
+    case _MAKE_VERSION_ID(PY_VERSION, PY_FLAGS): { \
         PyStatus status; \
         PYCONFIG_IMPL *config_impl = (PYCONFIG_IMPL *)config; \
-        status = PI_PyConfig_SetWideStringList(config, &config_impl->module_search_paths, num_paths, paths); \
+        status = dylib_python->PyConfig_SetWideStringList(config, &config_impl->module_search_paths, num_paths, paths); \
         config_impl->module_search_paths_set = 1; \
-        return PI_PyStatus_Exception(status) ? -1 : 0; \
+        return dylib_python->PyStatus_Exception(status) ? -1 : 0; \
     }
     /* Macro end */
 
-    switch (python_version) {
-        _IMPL_CASE(308, PyConfig_v38)
-        _IMPL_CASE(309, PyConfig_v39)
-        _IMPL_CASE(310, PyConfig_v310)
-        _IMPL_CASE(311, PyConfig_v311)
-        _IMPL_CASE(312, PyConfig_v312)
+    switch (version_id) {
+        _IMPL_CASE(308, 0, PyConfig_v38)
+        _IMPL_CASE(309, 0, PyConfig_v39)
+        _IMPL_CASE(310, 0, PyConfig_v310)
+        _IMPL_CASE(311, 0, PyConfig_v311)
+        _IMPL_CASE(312, 0, PyConfig_v312)
+        _IMPL_CASE(313, 0, PyConfig_v313)
+        _IMPL_CASE(313, 1, PyConfig_v313_GIL_DISABLED)
         default: {
             break;
         }
@@ -432,8 +477,13 @@ _pyi_pyconfig_set_module_search_paths(PyConfig *config, int python_version, int
 }
 
 int
-pyi_pyconfig_set_module_search_paths(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
+pyi_pyconfig_set_module_search_paths(PyConfig *config, const struct PYI_CONTEXT *pyi_ctx)
 {
+#if !defined(_WIN32)
+    /* Py_DecodeLocale / PyMem_RawFree in non-Windows codepath. */
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
+#endif
+
     /* TODO: instead of stitching together char strings and converting
      * them, we could probably stitch together wide-char strings directly,
      * as `home` field in config structure has already been converted. */
@@ -465,21 +515,21 @@ pyi_pyconfig_set_module_search_paths(PyConfig *config, const PYI_CONTEXT *pyi_ct
 #ifdef _WIN32
         module_search_paths_w[i] = pyi_win32_utf8_to_wcs(module_search_paths[i], NULL, 0);
 #else
-        module_search_paths_w[i] = PI_Py_DecodeLocale(module_search_paths[i], NULL);
+        module_search_paths_w[i] = dylib_python->Py_DecodeLocale(module_search_paths[i], NULL);
 #endif
         if (module_search_paths_w[i] == NULL) {
             /* Do not break; we need to initialize all elements */
             ret = -1;
         }
     }
-    if (ret < -1) {
+    if (ret != 0) {
         goto end; /* Conversion of at least one path failed */
     }
 
     /* Set */
     ret = _pyi_pyconfig_set_module_search_paths(
         config,
-        pyi_ctx->archive->python_version,
+        pyi_ctx,
         3,
         module_search_paths_w
     );
@@ -490,7 +540,7 @@ end:
 #ifdef _WIN32
         free(module_search_paths_w[i]);
 #else
-        PI_PyMem_RawFree(module_search_paths_w[i]);
+        dylib_python->PyMem_RawFree(module_search_paths_w[i]);
 #endif
     }
 
@@ -502,24 +552,29 @@ end:
  * Set program arguments (sys.argv).
  */
 static int
-_pyi_pyconfig_set_argv(PyConfig *config, int python_version, int argc, wchar_t **argv_w)
+_pyi_pyconfig_set_argv(PyConfig *config, const struct PYI_CONTEXT *pyi_ctx, int argc, wchar_t **argv_w)
 {
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
+    const int version_id = _MAKE_VERSION_ID(dylib_python->version, pyi_ctx->nogil_enabled);
+
     /* Macro to avoid manual code repetition. */
-    #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
-    case PY_VERSION: { \
+    #define _IMPL_CASE(PY_VERSION, PY_FLAGS, PYCONFIG_IMPL) \
+    case _MAKE_VERSION_ID(PY_VERSION, PY_FLAGS): { \
         PyStatus status; \
         PYCONFIG_IMPL *config_impl = (PYCONFIG_IMPL *)config; \
-        status = PI_PyConfig_SetWideStringList(config, &config_impl->argv, argc, argv_w); \
-        return PI_PyStatus_Exception(status) ? -1 : 0; \
+        status = dylib_python->PyConfig_SetWideStringList(config, &config_impl->argv, argc, argv_w); \
+        return dylib_python->PyStatus_Exception(status) ? -1 : 0; \
     }
     /* Macro end */
 
-    switch (python_version) {
-        _IMPL_CASE(308, PyConfig_v38)
-        _IMPL_CASE(309, PyConfig_v39)
-        _IMPL_CASE(310, PyConfig_v310)
-        _IMPL_CASE(311, PyConfig_v311)
-        _IMPL_CASE(312, PyConfig_v312)
+    switch (version_id) {
+        _IMPL_CASE(308, 0, PyConfig_v38)
+        _IMPL_CASE(309, 0, PyConfig_v39)
+        _IMPL_CASE(310, 0, PyConfig_v310)
+        _IMPL_CASE(311, 0, PyConfig_v311)
+        _IMPL_CASE(312, 0, PyConfig_v312)
+        _IMPL_CASE(313, 0, PyConfig_v313)
+        _IMPL_CASE(313, 1, PyConfig_v313_GIL_DISABLED)
         default: {
             break;
         }
@@ -537,11 +592,11 @@ _pyi_pyconfig_set_argv(PyConfig *config, int python_version, int argc, wchar_t *
  * of wide-char strings, we can directly pass it into Python's
  * configuration structure. */
 int
-pyi_pyconfig_set_argv(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
+pyi_pyconfig_set_argv(PyConfig *config, const struct PYI_CONTEXT *pyi_ctx)
 {
     return _pyi_pyconfig_set_argv(
         config,
-        pyi_ctx->archive->python_version,
+        pyi_ctx,
         pyi_ctx->argc,
         pyi_ctx->argv_w
     );
@@ -555,8 +610,9 @@ pyi_pyconfig_set_argv(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
  * function, which accounts for the locale/encoding that was set up
  * during pre-initialization of Python interpreter. */
 int
-pyi_pyconfig_set_argv(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
+pyi_pyconfig_set_argv(PyConfig *config, const struct PYI_CONTEXT *pyi_ctx)
 {
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
     char *const *argv;
     wchar_t **argv_w;
     int argc;
@@ -582,20 +638,17 @@ pyi_pyconfig_set_argv(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
 
     /* Convert */
     for (i = 0; i < argc; i++) {
-        argv_w[i] = PI_Py_DecodeLocale(argv[i], NULL);
+        argv_w[i] = dylib_python->Py_DecodeLocale(argv[i], NULL);
         if (argv_w[i] == NULL) {
-            /* Do not break; we need to initialize all elements */
             ret = -1;
+            goto end;
         }
     }
-    if (ret < -1) {
-        goto end; /* Conversion of at least one arg failed */
-    }
 
     /* Set */
     ret = _pyi_pyconfig_set_argv(
         config,
-        pyi_ctx->archive->python_version,
+        pyi_ctx,
         argc,
         argv_w
     );
@@ -603,7 +656,7 @@ pyi_pyconfig_set_argv(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
 end:
     /* Cleanup */
     for (i = 0; i < argc; i++) {
-        PI_PyMem_RawFree(argv_w[i]);
+        dylib_python->PyMem_RawFree(argv_w[i]);
     }
     free(argv_w);
 
@@ -617,11 +670,15 @@ end:
  * Set run-time options.
  */
 int
-pyi_pyconfig_set_runtime_options(PyConfig *config, int python_version, const PyiRuntimeOptions *runtime_options)
+pyi_pyconfig_set_runtime_options(PyConfig *config, const struct PYI_CONTEXT *pyi_ctx, const struct PyiRuntimeOptions *runtime_options)
 {
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
+    const int version_id = _MAKE_VERSION_ID(dylib_python->version, pyi_ctx->nogil_enabled);
+
+    /* *** Common options *** */
     /* Macro to avoid manual code repetition. */
-    #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
-    case PY_VERSION: { \
+    #define _IMPL_CASE(PY_VERSION, PY_FLAGS, PYCONFIG_IMPL) \
+    case _MAKE_VERSION_ID(PY_VERSION, PY_FLAGS): { \
         PyStatus status; \
         PYCONFIG_IMPL *config_impl = (PYCONFIG_IMPL *)config; \
         /* Extend the isolated config, which leaves site_import and write_bytecode on */ \
@@ -641,16 +698,16 @@ pyi_pyconfig_set_runtime_options(PyConfig *config, int python_version, const Pyi
         config_impl->dev_mode = runtime_options->dev_mode; \
         /* Set W-flags, if available */ \
         if (runtime_options->num_wflags) { \
-            status = PI_PyConfig_SetWideStringList(config, &config_impl->warnoptions, runtime_options->num_wflags, runtime_options->wflags); \
-            if (PI_PyStatus_Exception(status)) { \
+            status = dylib_python->PyConfig_SetWideStringList(config, &config_impl->warnoptions, runtime_options->num_wflags, runtime_options->wflags); \
+            if (dylib_python->PyStatus_Exception(status)) { \
                 return -1; \
             } \
         } \
         /* Set X-flags, if available. Note that this is just pass-through that allows options to show up in sys._xoptions;
          * for example, for -Xutf8 or -Xdev to take effect, we need to explicitly parse them and modify PyConfig fields. */ \
         if (runtime_options->num_xflags) { \
-            status = PI_PyConfig_SetWideStringList(config, &config_impl->xoptions, runtime_options->num_xflags, runtime_options->xflags); \
-            if (PI_PyStatus_Exception(status)) { \
+            status = dylib_python->PyConfig_SetWideStringList(config, &config_impl->xoptions, runtime_options->num_xflags, runtime_options->xflags); \
+            if (dylib_python->PyStatus_Exception(status)) { \
                 return -1; \
             } \
         } \
@@ -666,12 +723,14 @@ pyi_pyconfig_set_runtime_options(PyConfig *config, int python_version, const Pyi
     }
     /* Macro end */
 
-    switch (python_version) {
-        _IMPL_CASE(308, PyConfig_v38)
-        _IMPL_CASE(309, PyConfig_v39)
-        _IMPL_CASE(310, PyConfig_v310)
-        _IMPL_CASE(311, PyConfig_v311)
-        _IMPL_CASE(312, PyConfig_v312)
+    switch (version_id) {
+        _IMPL_CASE(308, 0, PyConfig_v38)
+        _IMPL_CASE(309, 0, PyConfig_v39)
+        _IMPL_CASE(310, 0, PyConfig_v310)
+        _IMPL_CASE(311, 0, PyConfig_v311)
+        _IMPL_CASE(312, 0, PyConfig_v312)
+        _IMPL_CASE(313, 0, PyConfig_v313)
+        _IMPL_CASE(313, 1, PyConfig_v313_GIL_DISABLED)
         default: {
             break;
         }
@@ -687,11 +746,13 @@ pyi_pyconfig_set_runtime_options(PyConfig *config, int python_version, const Pyi
  * Pre-initialize python interpreter.
  */
 int
-pyi_pyconfig_preinit_python(const PyiRuntimeOptions *runtime_options)
+pyi_pyconfig_preinit_python(const struct PyiRuntimeOptions *runtime_options, const struct PYI_CONTEXT *pyi_ctx)
 {
+    const struct DYLIB_PYTHON *dylib_python = pyi_ctx->dylib_python;
     PyPreConfig_Common config;
+    PyStatus status;
 
-    PI_PyPreConfig_InitIsolatedConfig((PyPreConfig *)&config);
+    dylib_python->PyPreConfig_InitIsolatedConfig((PyPreConfig *)&config);
 
     config.utf8_mode = runtime_options->utf8_mode;
     config.dev_mode = runtime_options->dev_mode;
@@ -700,6 +761,6 @@ pyi_pyconfig_preinit_python(const PyiRuntimeOptions *runtime_options)
     config.configure_locale = 1;
 
     /* Pre-initialize */
-    PyStatus status = PI_Py_PreInitialize((const PyPreConfig *)&config);
-    return PI_PyStatus_Exception(status) ? -1 : 0;
+    status = dylib_python->Py_PreInitialize((const PyPreConfig *)&config);
+    return dylib_python->PyStatus_Exception(status) ? -1 : 0;
 }
